      program acrmf
c   ACRM modified for forest, A. Kuusk, December 2012
c   The format of input file is changed, see the readme.txt.
c
c   Two-layer reflectance model, A. Kuusk, December 2002
c   A. Kuusk, Tartu Observatory, andres@aai.ee
c
c  A. Kuusk, A two-layer canopy reflectance model,
c  J.Q.S.R.T. 2001, 71(1):1-9.
c
c  Inversion of the model is similar to the inversion of the 
c  forest reflectance model frt, version 09.2002:
c   Kuusk, A. and Nilson, T. A directional multispectral forest 
c   reflectance model, Remote Sens. Environ., 2000, 72(2): 
c   244-252.
c
c  Usage:  ./acrmf infile outfile
c
      include 'acrmpar.h'
      include 'nch.h'
      include 'ifunc.h'
      include 'spec.h'
c
      logical lig
*     character outx*12, xfs*7
      character*2 trl
      character*6 mv
      character*30 infile, oufile, fname
      character*30 frefr, fsky, fprice, fcoef1, fcoef2, fcoef3
      character*10 lmod1, lmod2
      character*30 dname
c
      dimension fcoef1(ncomp0), fcoef2(ncomp0), fcoef3(nclmax, ncomp0)
      dimension ncomp3(nclmax)
c
      dimension thsarr(nsunmax), bangs(nsunmax)
      dimension wlarr(nspchnl)
      dimension thknot(nangls), p14(nknots)
      dimension zctst(ncub), acztst(ncub)
c
      dimension th1j(nreflval), baj(nreflval), th2j(nreflval),
     & phij(nreflval), wlgth(nreflval)
*    & , bqj(nreflval)
      dimension bqjarr(nspchnl), drhoarr(nspchnl)
c
      dimension llxx(nvar), xmin(nvar), xmax(nvar), xx(nvar), xa(nvar), 
     &          xb(nvar), xxf(nvar), xdelt(nvar), xe(nvar)
c     xx - the result of iteration in the inversion
c     xa, xb - scaling prameters
c     xxf - scaled and reordered vector of parameters
c     xe - expert estimate for the inversion
c     xdelt - variation of model parameters subject to estimation
c
      character*20 tex
      character*30 tjob
      dimension tex(40), tjob(6)
      dimension parr(nvar+1, nvar)
c
      common /par/ zeps, tol, tiny, alphopt, betopt, gammopt, 
     &      ier, ncount, nfmax, itmax, itbr, nbrak, ilo
c
      data trl/'# '/, lig/.false./
*     data dr/1.745329251994330d-2/, pi/3.14159265358979d0/
      data eps2/1.745d-2/, dthetv/.1d0/, thtv0/0.d0/
c
      data tex/' ', ' ', ' ', ' ', ' ', ' ',
     & ' ', ' ',  ' ', ' ', ' ', ' ', ' ',
     & ' c1, % of SLW', ' c2, % of SLW', ' c3, % of SLW',
     & ' c4, % of SLW', ' c5, % of SLW', ' c6, % of SLW',
     & ' c7, % of SLW', ' c8, % of SLW', ' c9, % of SLW',
     & ' c10, % of SLW', ' D_cell, µm', ' i-cell air', 
     & ' leaf struct. par.', ' ', ' shoot size, m', ' LAI', 
     & ' leaf relat. size', ' clmp', ' szz', ' eln', ' thm', 
     & ' n_ratio', ' SLW', ' s1_soil', ' s2', ' s3', ' s4'/
      data tjob /' - directional reflectance', 
     & ' - spectrum', ' - angular distribution',
     & ' - several Sun angles', ' - inversion, relative errors', 
     & ' - inversion, absolute errors'/
c
*     call fptrap(1)
c
      pi = acos(-1.d0)
      dr = pi/180.d0
c
      iwr    = 1
      ire    = 2
      ncount = 0
      ier    = 0
      iter   = 1
      pkhair = .1d0
      dcell0 = 80.d0
c
c   Initialisation: the knots and weigths of Gauss-Legendre quadrature
      ntstz   = nknots - 1
      x1      = 0.d0
      x2      = pi/2.d0
      call gauleg(x1, x2, zctst, acztst, ntstz)
      thknot(1) = 0.d0
      p14(1)    = 0.d0
      do jth = 2, nknots
         thknot(jth) = zctst(jth - 1)
         p14(jth)    = cos(thknot(jth))*acztst(jth - 1)
      enddo
c
      do jc = 1, ncomp0
         xe(jc)     = 0.d0
         xmin(jc)   = 0.d0
         xmax(jc)   = 0.d0
         xdelt(jc)  = 0.d0
         fcoef1(jc) = ' '
         fcoef2(jc) = ' '
         do ncl = 1, nclmax
            ncomp3(ncl)     = 0
            fcoef3(ncl, jc) = ' '
         enddo
      enddo
c
      call getarg(1, infile)
      call getarg(2, oufile)
      if (infile .eq. ' ') then
         write (0, '(a)')  ' Name of input file '
         read (*, '(a)') infile
      endif
      if (oufile .eq. ' ') then
         write (0, '(a)')  ' Name of output file '
         read (*, '(a)') oufile
      endif
c
c                                    Read header and IJOB
      fname = infile
      open (ire, file = fname, status = 'old', err = 99)
      rewind ire
           jl = 1
      read (ire, *, err = 98) dname
      write (0, '(/2a)') ' Data set: ', dname
           jl = jl + 1
      read (ire, *, err = 98)
           jl = jl + 1
      read (ire, *, err = 98) frefr, fsky
           jl = 22
      read (ire, '(8(/))', err = 97)
      read (ire, *, err = 98) lmod2
      read (ire, *, err = 98) ncomp2
         nl = ncomp2 + 8
         if (lmod2 .eq. 'liberty') nl = ncomp2 + 10
      do jr = 1, nl
         jl = jl + 1
         read (ire, *, err = 98)
      enddo
      read (ire, *, err = 98) lmod1
      read (ire, *, err = 98) ncomp1
         nl = ncomp1 + 6
         if (lmod1 .eq. 'liberty') nl = ncomp1 + 8
      do jr = 1, nl
         jl = jl + 1
         read (ire, *, err = 98)
      enddo
      read (ire, *, err = 98) ijob
      write (0, '(a, i3, a)') ' ijob = ', ijob, tjob(ijob+1)
*     close (ire)
c
c                                Read vegetation parameters
c                                the upper layer
*     open (ire, file = fname, status = 'old', err = 99)
      rewind ire
      jl = 4
      read (ire, '(3(/))', err = 98)
      j1 = 29
      j2 = j1 + 7
      do jx = j1, j2
         jl = jl + 1
         if (ijob .gt. 3) then
            read (ire, *, err = 98) xe(jx),
     &                 xmin(jx), xmax(jx), xdelt(jx)
         else
            read (ire, *, err = 98) xe(jx)
         endif
      enddo
        jl = jl + 1
      read (ire, *, err = 98) lmod2
        jl = jl + 1
      read (ire, *, err = 98) ncomp2
      do jc = 1, ncomp2
         jx = j2 + jc
         jl = jl + 1
         read (ire, *, err = 98) xe(jx),
     &            xmin(jx), xmax(jx), xdelt(jx), fcoef2(jc)
      enddo
      if (lmod2 .eq. 'liberty') then
         j1 = 47
         j2 = j1 + 2
      else
         j1 = 49
         j2 = j1
      endif
      do jx = j1, j2
         jl = jl + 1
         if (ijob .gt. 3) then
            read (ire, *, err = 98) xe(jx),
     &              xmin(jx), xmax(jx), xdelt(jx)
         else
            read (ire, *, err = 98) xe(jx)
         endif
      enddo
c                                           the lower layer
         j1 = 50
         j2 = j1 + 6
      do jx = j1, j2
         jl = jl + 1
         if (ijob .gt. 3) then
            read (ire, *, err = 98) xe(jx),
     &              xmin(jx), xmax(jx), xdelt(jx)
         else
            read (ire, *, err = 98) xe(jx)
         endif
      enddo
        jl = jl + 1
      read (ire, *, err = 98) lmod1
        jl = jl + 1
      read (ire, *, err = 98) ncomp1
      do jc = 1, ncomp1
         jx = j2 + jc
         jl = jl + 1
         read (ire, *, err = 98) xe(jx),
     &            xmin(jx), xmax(jx), xdelt(jx), fcoef1(jc)
      enddo
      if (lmod1 .eq. 'liberty') then
         j1 = 67
         j2 = j1 + 2
      else
         j1 = 69
         j2 = j1
      endif
      do jx = j1, j2
         jl = jl + 1
         if (ijob .gt. 3) then
            read (ire, *, err = 98) xe(jx),
     &                 xmin(jx), xmax(jx), xdelt(jx)
         else
            read (ire, *, err = 98) xe(jx)
         endif
      enddo
c                                Read soil parameters
      jl = jl + 1
      read (ire, *, err = 98) fprice, thssr
      j1 = 70
      j2 = j1 + 3
      do jx = j1, j2
         jl = jl + 1
         if (ijob .gt. 3) then
            read (ire, *, err = 98) xe(jx),
     &              xmin(jx), xmax(jx), xdelt(jx)
         else
            read (ire, *, err = 98) xe(jx)
         endif
      enddo
      thss  = thssr*dr
c
        jl = jl + 1
      read (ire, *, err = 98) ijob
c
c                    ! Read Sun angles, wavelengths and view angles
        jl = jl + 1
      read (ire, *, err = 98) nsun, nchnl, dwl
      if (ijob .le. 3 .and. ijob .ne. 1) nchnl = 1
      if (ijob .lt. 3) nsun  = 1
        jl = jl + 1
      read (ire, *, err = 98) (thsarr(js), js = 1, nsun)
        jl = jl + 1
      read (ire, *, err = 98) (bangs(js), js = 1, nsun)
c
      if (dwl .le. 0.d0) then
        jl = jl + 1
         read (ire, *, err = 98) (wlarr(jw), jw = 1, nchnl)
      else
        jl = jl + 1
         read (ire, *, err = 98) wlarr(1)
      endif
c
        jl = jl + 1
      read (ire, *, err = 98) thetvs, dthtvs, phivs
c
      if (ijob .gt. 3) then
c                           ! Read minimization parameters
           jl = jl + 1
         read (ire, *, err = 98) mv
           jl = jl + 1
         read (ire, *, err = 98) nfmax, itmax, itbr, nbrak
           jl = jl + 1
         read (ire, *, err = 98) zeps, tol, tiny, ftol
           jl = jl + 1
         read (ire, *, err = 98) alphopt, betopt, gammopt, dxopt
           jl = jl + 1
         read (ire, *, err = 98) nxx, at, lig
c                           ! lig = .true.  - use the previous results 
c                                             as initial guess
           jl = jl + 1
         read (ire, *, err = 98) (llxx(jx), jx = 1, nxx)
           jl = jl + 1
         read (ire, *, err = 98) nref
c
c
c                                     ! Read reflectance data
         do jr = 1, nref
            jl = jl + 1
            read (ire, *, err = 98) 
     &         wlgth(jr), bqjarr(jr), drhoarr(jr),
     &         th1j(jr),  phij(jr), th2j(jr), baj(jr)
            if (drhoarr(jr) .lt. 0.d0) drhoarr(jr) = 0.1d0*bqjarr(jr)
         enddo
      endif
c
      close (ire)
c
c                                      Read tabulated spectra
      ncl = 0
      call rspec(frefr, fprice, fcoef1, ncomp1, 
     & fcoef2, ncomp2, fcoef3, ncomp3, ncl)
c
      fname = fsky
      open (7, file = fname, status = 'old', err = 99)
         rewind 7
         read (7, '(/)', err = 98)
         do jl = 1, nspchnl
            read (7, *, err = 98)
     &      wlgthd, alpl(jl), gamml(jl)
         enddo
      close (7)
c
      write (0, '(2a)') ' Output file: ', oufile
c
*     if (ijob .gt. 3) then
*        xfs    = oufile
*        outx   = xfs//'0.dat'
*        if (lig) then
*           fname = outx
*           open (7, file = fname, status = 'old', err = 99)
*           rewind 7
*           jl = 1
*           read (7, '(d27.17)', err = 98) (xe(jx), jx = 1, nvar)
*           close (7)
*        endif
*     endif
c
      open (iwr, file = oufile, status = 'replace')
*     open (iwr, file = oufile, status = 'new')
      rewind iwr
c
c                                               ! Print input data
      write (iwr, '(a/2a)') trl, trl, 
     & '  Reflectance of a two-layer canopy by A. Kuusk, ACRMf_12.2012'
      write (iwr, '(a/a, 5x, a)') trl, 
     & '##  Input parameters:', dname
c
      if (ijob .eq. 0) then
         nsun  = 1
         nchnl = 1
         write (iwr, '(a, i3, a)') '##  ijob = ', ijob,
     & '  - a single reflectance value in the given direction'
      else if (ijob .eq. 1) then
         nsun  = 1
         write (iwr, '(a, i3, a)') '##  ijob = ', ijob,
     & '  - the reflectance spectrum'
      else if (ijob .eq. 2) then
         nsun  = 1
         nchnl = 1
         write (iwr, '(a, i3, a)') '##  ijob = ', ijob,
     & '  - the angular distribution of reflectance'
      else if (ijob .eq. 3) then
         nchnl = 1
         write (iwr, '(a, i3, a)') '##  ijob = ', ijob,
     & '  - several Sun zenith angles'
      else if (ijob .eq. 4) then
         write (iwr, '(a, i3, a)') '##  ijob = ', ijob,
     & '  - inversion, relative differences and expert estimate'
      else if (ijob .eq. 5) then
         write (iwr, '(a, i3, a)') '##  ijob = ', ijob,
     & '  - inversion, absolute differences'
      else
         write (iwr, '(a, i3)') 'Invalid IJOB, ijob = ', ijob
         write (0, '(a, i3)') 'Invalid IJOB, ijob = ', ijob
         stop 'ACRM'
      endif
c
      write (iwr, '(a/a/a/a/a, 2x, 3a20)') trl,
     & '#   Files of refractive index,',
     & '#   of McCartney functions for sky fluxes,',
     & '#   and of Price'' vectors:', 
     & trl, frefr, fsky, fprice
      write (iwr, '(2a, 2x, f6.1)') trl,
     & '  The sun zenith of soil reflectance', thssr
c
      if (ijob .le. 3) then
         write (iwr, '(a/a, 18x, a, 4x, a)') 
     &   trl, '# ***', ' Upper layer', 'Lower layer'
      else
         write (iwr, '(a/a, 11x, a)') trl, '# *** Upper layer',
     &   'x0      x_min    x_max      dx'
      endif
      jx = 28
      jt = 28
      j2 = 3
      do jr = 1, j2
         jx = jx + 1
         jt = jt + 1
         j4 = jx + 21
         if (ijob .gt. 3) then
            write (iwr, '(a, 1x, a20, 23f9.3)')
     & trl, tex(jt), xe(jx), xmin(jx), xmax(jx), xdelt(jx)
         else
            write (iwr, '(a, 1x, a20, 23f12.3)')
     & trl, tex(jt), xe(jx), xe(j4)
         endif
      enddo
         jx = jx + 1
         jt = jt + 1
         if (ijob .gt. 3) then
            write (iwr, '(a, 1x, a20, 23f9.3)')
     & trl, tex(jt), xe(jx), xmin(jx), xmax(jx), xdelt(jx)
         else
            write (iwr, '(a, 1x, a20, 23f12.3)')
     & trl, tex(jt), xe(jx)
         endif
c
      jx = 32
      jt = 32
      j2 = 4
      do jr = 1, j2
         jx = jx + 1
         jt = jt + 1
         j4 = jx + 20
         if (ijob .gt. 3) then
            write (iwr, '(a, 1x, a20, 23f9.3)')
     & trl, tex(jt), xe(jx), xmin(jx), xmax(jx), xdelt(jx)
         else
            write (iwr, '(a, 1x, a20, 23f12.3)')
     & trl, tex(jt), xe(jx), xe(j4)
         endif
      enddo
c
      if (ijob .le. 3) then
         write (iwr, '(a, 2x, a12, 8x, a12, a12)') 
     & trl, 'Leaf models: ', lmod2, lmod1
         write (iwr, '(a, 2x, a21, i10, i12)')
     & trl, '# of leaf components:', ncomp2, ncomp1
      else
         write (iwr, '(a, 2x, a12, 8x, a12, i6, a)')
     & trl, 'Leaf model: ', lmod2, ncomp2, ' components'
      endif
c
      if (ijob .le. 3) then
         ncmax = max(ncomp1, ncomp2)
         jt = 13
         do jc = 1, ncmax
           jx = jx + 1
           jt = jt + 1
           j4 = jx + 20
            write (iwr, '(a, 5x, 10a18)')
     &      trl, fcoef2(jc), fcoef1(jc)
            write (iwr, '(a, 1x, a20, 23f12.3)')
     &      trl, tex(jt), xe(jx), xe(j4)
         enddo
c
c print LIBERTY parameters 
         if ((lmod1 .eq. 'liberty') .or. (lmod2 .eq. 'liberty')) then
            jt = 22
            j1 = 46
            j2 = j1 + 1
            write (iwr, '(a, 1x, a20, 23f12.4)')
     &       trl, tex(jt), xe(jx), xe(j4)
         endif
c print PROSPECT parameter N
            jt = 26
            j1 = 49
            j4 = j1 + 20
            write (iwr, '(a, 1x, a20, 23f12.4)')
     &       trl, tex(jt), xe(j1), xe(j4)
c
      else
         jt = 13
         do jc = 1, ncomp2
           jx = jx + 1
           jt = jt + 1
           write (iwr, '(a, 5x, 10a18)')
     &       trl, fcoef2(jc)
           write (iwr, '(a, 1x, a20, 23f9.3)')
     &       trl, tex(jt), xe(jx), xmin(jx), xmax(jx), xdelt(jx)
         enddo
c
c print LIBERTY parameters 
         if (lmod2 .eq. 'liberty') then
            jt = 22
            j1 = 46
            j2 = j1 + 1
            write (iwr, '(a, 1x, a20, 23f9.4)')
     &        trl, tex(jt), xe(jx), xmin(jx), xmax(jx), xdelt(jx)
         endif
c print PROSPECT parameter N
            jt = 26
            j1 = 49
            write (iwr, '(a, 1x, a20, 23f9.4)') trl, 
     &        tex(jt), xe(j1), xmin(j1), xmax(j1), xdelt(j1)
c
         write (iwr, '(a)') '# *** Lower layer'
         j1 = 50
         j2 = j1 + 6
         jt = 28
         do jx = j1, j2
            jt = jt + 1
            if (jt .eq. 32) jt = jt + 1
            write (iwr, '(a, 1x, a20, 23f9.3)')
     &        trl, tex(jt), xe(jx), xmin(jx), xmax(jx), xdelt(jx)
         enddo
c
         write (iwr, '(a, 2x, a12, 8x, a12, i6, a)')
     &     trl, 'Leaf model: ', lmod1, ncomp1, ' components'
         jt = 13
         jx = j2
         do jc = 1, ncomp1
              jx = jx + 1
              jt = jt + 1
            write (iwr, '(a, 5x, 10a18)') trl, fcoef1(jc)
            write (iwr, '(a, 1x, a20, 23f9.3)')
     & trl, tex(jt), xe(jx), xmin(jx), xmax(jx), xdelt(jx)
         enddo
c
c print LIBERTY parameters 
         if (lmod1 .eq. 'liberty') then
            j1 = 67
            jt = 23
            j2 = j1 + 1
      do jx = j1, j2
         jt = jt + 1
            write (iwr, '(a, 1x, a20, 23f9.4)')
     &        trl, tex(jt), xe(jx), xmin(jx), xmax(jx), xdelt(jx)
      enddo
         endif
c print PROSPECT parameter N
            jx = 69
            jt = 26
            write (iwr, '(a, 1x, a20, 23f9.4)') trl, 
     &        tex(jt), xe(jx), xmin(jx), xmax(jx), xdelt(jx)
      endif
c
      jt = 36
      j1 = 70
      j2 = j1 + 3
      do jx = j1, j2
         jt = jt + 1
         if (ijob .gt. 3) then
            write (iwr, '(a, 1x, a20, 23f9.4)')
     & trl, tex(jt), xe(jx), xmin(jx), xmax(jx), xdelt(jx)
         else
            write (iwr, '(a, 1x, a20, 23f12.4)')
     & trl, tex(jt), xe(jx)
         endif
      enddo
c
      write (iwr, '(a, 1x, a20, 23f9.1)')
     & trl, 'Sun zenith(s)', (thsarr(js), js = 1, nsun)
      write (iwr, '(a, 3x, a20, 23f9.3)')
     & trl, 'b_Ångström   ', (bangs(js), js = 1, nsun)
c
      write (iwr, '(a)') '#  '
c
c
      if (ijob .gt. 3) then
         write (iwr, '(a)') 
     & '# lmbda      refl     std_r    thv    phi    ths   b_Ångstr'
c                  !  print input reflectances and transmittances
         do jr = 1, nref
            write (iwr, '(a, f7.1, 2f10.4, 3f7.1, f8.3)')
     &         trl, wlgth(jr), bqjarr(jr), drhoarr(jr),
     &         th1j(jr),  phij(jr), th2j(jr), baj(jr)
         enddo
         write (iwr, '(a)') trl
      endif
      close (iwr)
c
c  ******************                    ! Start of computations
c
c                                        ! Check wavelength range
      do jw = 1, nchnl
         if (dwl .gt. 0.d0) wlarr(jw) = wlarr(1) + (jw - 1)*dwl
         rlambda   = wlarr(jw)
         if ((rlambda .lt. 400.d0) .or. (rlambda .gt. 2400.d0)) 
     &     goto 88
      enddo
c
c                                 ! copy fixed model parameters into COMMON
      i_job   = ijob
c
      thetv   = thetvs*dr
      if (abs(thetv) .lt. dr) then
         p_hi = 0.d0
      else
         p_hi = phivs*dr
      endif
      phi     = p_hi
      p_khair = pkhair
      l_mod1  = lmod1
      l_mod2  = lmod2
      n_cmp1  = ncomp1
      n_cmp2  = ncomp2
      t_hss   = thss
      d_cel0  = dcell0
      a_t     = at
c
      do jx = 1, nvar
         x_e(jx) = xe(jx)
         xx(jx)  = xe(jx)
      enddo
c
      do js = 1, nknots
         t_hknot(js)  = thknot(js)
         p_14(js)      = p14(js)
      enddo
c
      if (ijob .gt. 3) then
c                                scale the vector of parameters
         do i = 1, nxx
            li        = llxx(i)
            xa(li)    = (xmax(li) + xmin(li))*.5d0
            xb(li)    = 10.d0/(xmax(li) - xmin(li))
            xxf(i)    = (xx(li) - xa(li))*xb(li)
         enddo
      endif
c                    ! copy variable model parameters into COMMON
      do jx = 1, nvar
         l_lxx(jx)  = llxx(jx)
         x_x(jx)    = xx(jx)
         x_a(jx)    = xa(jx)
         x_b(jx)    = xb(jx)
         x_delt(jx) = xdelt(jx)
      enddo
c
      jv = 1
      t_hs = thsarr(1)*dr
      b_ang = bangs(1)
      t_hv = thetvs*dr
      r_lmbda = wlarr(1)
c
      if (ijob .le. 3) then
         open (iwr, file = oufile, status = 'old')
         rewind iwr
         do j = 1, 43
            read (iwr, '(1x)', end = 51)
         enddo
c
51       continue
            backspace iwr
         write (iwr, '(a/a)') trl, '# *** Results:'
      endif
c
c                ijob = 0 -- single value of reflectance
c
      if (ijob .eq. 0) then
c           ! reflectance value is calculated in the function FUNC(..,..)
c           ! nxx variable model parameters are in the scaled vector xxf,
c           ! the fixed model parameters and tabulated functions
c           ! are in COMMON areas
         fff  = func(nxx, xxf)
         sq   = s_q
         bq   = sq*(b_c1 + b_s1) + b_dplants + b_dsoil
c
         write (iwr, '(2(/a, f6.1)/a, f6.2/a, 2f8.1/a, f8.4)')
     & ' Wavelength, nm ', rlambda,
     & ' Sun zenith, deg', thsarr(1),
     & ' S''/Q           ', sq,
     & ' View zenith and azimuth', thetv/dr, phi/dr,
     & ' Reflectance  ', bq
c
         write (0, '(2(/a, f6.1)/a, f6.2/a, 2f8.1/a, f8.4)')
     & ' Wavelength, nm ', rlambda,
     & ' Sun zenith, deg', thsarr(1),
     & ' S''/Q           ', sq,
     & ' View zenith and azimuth', thetv/dr, phi/dr,
     & ' Reflectance  ', bq
c
         close (iwr)
      endif
c
c                ******************  Loop to calculate spectrum
c
      if (ijob .eq. 1) then
         do jw = 1, nchnl
            rlambda = wlarr(jw)
            r_lmbda = rlambda
            fff  = func(nxx, xxf)
            sq   = s_q
            bq   = sq*(b_c1 + b_s1) + b_dplants + b_dsoil
            ggv  = g_gv
c
*           if (jw .eq. 1) 
*    &      write (iwr, '(a/a, f6.1/a, 2f8.1/a/a/a)') trl,
*    &      '# Sun zenith, deg', thsarr(1),
*    &      '# View zenith and azimuth', thetv/dr, phi/dr, trl,
*    &      '# lmbda, nm;  reflectance; S''/Q', trl
*           write (iwr, '(f6.1, f10.4, f10.2)') rlambda, bq, sq
*           if (mod(jw, 10) .eq. 0) print *, rlambda, bq
c
            if (jw .eq. 1) 
     &      write (iwr, '(a/a, f6.1/a, 2f8.1/a, 68x, a/a/a)') trl,
     &      '# Sun zenith, deg  ', thsarr(1),
     &      '# View zenith and azimuth, deg', thetv/dr, phi/dr, trl,
     &      'leaves',
     & '#lmbda, nm; refl;    rc1;      rcd;      rs1;      rsd;     S''/
     &Q;     refl1;    trans1;   refl2;    trans2', trl
            write (iwr, '(f6.1, 10f10.4)') 
     &      rlambda, bq, b_c1, b_dplants, b_s1, b_dsoil, sq,
     &      r_efl1, t_ran1, r_efl2, t_ran2
            if (mod(jw, 10) .eq. 0) print *, rlambda, bq
         enddo
         close (iwr)
      endif
c
c               ************  Loop to calculate angular distribution
c
      if (ijob .eq. 2) then
         thtv00  = 80.d0
         nview   = int(2.d0*thtv00/dthtvs) + 1
         dthetv  = dthtvs*dr
         thtv0   = -(thtv00/dthtvs + 1)*dthetv
         print *, nview, ' view angles,    phi = ', phivs
c
         do jv = 1, nview
            thetv  = thtv0 + jv*dthetv
            if (abs(thetv) .lt. eps2) then
               thetv = 0.d0
               p_hi  = 0.d0
            else
               p_hi  = phivs*dr
            endif
            if (thetv .gt. 0.d0) then
               if (p_hi .gt. 0.d0) then
                  p_hi = pi - p_hi
               else
                  p_hi = pi
               endif
            endif
c
            t_hv = abs(thetv)
            phi  = p_hi
            fff  = func(nxx, xxf)
            sq   = s_q
            bq   = sq*(b_c1 + b_s1) + b_dplants + b_dsoil
            ggv2 = g_gv
            gfr2v = g_fr2v
c
*           if (jv .eq. 1) 
*    &      write (iwr, '(a, 2(/a, f6.1)/a, f6.2/a, f6.1/a/a/a)') trl,
*    &      '# Wavelength, nm   ', rlambda,
*    &      '# Sun zenith, deg  ', thsarr(1),
*    &      '# S''/Q             ', sq,
*    &      '# View azimuth, deg',  phi/dr, trl,
*    &      '# thv; reflectance; G_v', trl
*           write (iwr, '(f6.1, 2f10.4)') thetv/dr, bq, ggv
c
            if (jv .eq. 1) 
     &      write 
     &    (iwr, '(a, 2(/a, f6.1)/a, f6.2/a, f6.1/a, 4f9.4, /a/a)') 
     &    trl, '# Wavelength, nm   ', rlambda,
     &    '# Sun zenith, deg  ', thsarr(1),
     &    '# S''/Q             ', sq,
     &    '# View azimuth, deg',  phi/dr, 
     &    '# r_l1, t_l1, r_l2, t_l2: ', r_efl1, t_ran1, r_efl2, t_ran2,
     &    '# thv; refl; rc1, rcd, rs1, rsd, G2_v, gfr2_v', trl
          write (iwr, '(f6.1, 7f10.4)') 
     &    thetv/dr, bq, b_c1, b_dplants, b_s1, b_dsoil, ggv2, gfr2v
         enddo
         close (iwr)
      endif
c
c              ************ ijob = 3  -- several Sun angles
c
      if (ijob .eq. 3) then
         do js = 1, nsun
            t_hs = thsarr(js)*dr
            b_ang = bangs(js)
            fff  = func(nxx, xxf)
            sq   = s_q
            bq   = sq*(b_c1 + b_s1) + b_dplants + b_dsoil
c
            if (js .eq. 1)
     &      write (iwr, '(a/a, f6.1/a, 2f6.2/a/a)') trl,
     &      '# Wavelength, nm ', rlambda,
     &      '# View zenith and azimuth', thetv/dr, phi/dr,
     &      '# ths; reflectance;   S''/Q', trl
c
            write (iwr, '(f6.1, f10.4, f10.2)') thsarr(js), bq, sq
         enddo
         close (iwr)
      endif
c
c              ********************             ! Run inversion
      if (ijob .gt. 3) then
         fname = 'flow.dat'
         open (4, file = fname, status = 'old', err = 99)
         rewind 4
         jl = 1
c
11       continue
         if (ier .eq. 1) then
            close (4)
            goto 2
         else
            open (iwr, file = oufile, status = 'old')
               do j = 1, 4000
                  read (iwr, '(1x)', end = 61)
               enddo
61             continue
            backspace iwr
c
            write (iwr, '(a/a, i6, 3(a, i4))') trl, 
     &       '# nfmax = ', nfmax, '      itmax = ', itmax, 
     &       '     itbr  = ', itbr, '      nbrak = ', nbrak
            if (lig) then
               write (iwr, 202) nxx, nref, at, (llxx(i), i = 1, nxx)
202   format('# n     = ', i6, 6x, 'nref  = ', i4, 5x, 'at    = ', 
     & g8.3, 2x, 'lx = TRUE' /'# ll = ', 15i4)
            else
               write (iwr, 209) nxx, nref, at, (llxx(i), i = 1, nxx)
209   format('# n     = ', i6, 6x, 'nref  = ', i4, 5x, 'at  = ', 
     & g8.3, 4x, 'lx = FALSE'/'# ll = ', 15i4)
            endif
            close (iwr)
c
9           continue
            read (4, *, err = 98) jj
              jl = jl + 1
            goto (1, 2, 2, 2, 2, 2, 2, 2) jj
            goto 9
c
*1     print *, 'minimiseerimisprogramm?'
*      read (4, '(a)') mv
c
c              *****************************   ! The start of computations
1           continue
c
            i_job = ijob
            n_ref = nref
            do jr = 1, nref
               t_h1j(jr)    = th1j(jr)*dr
               b_aj(jr)     = baj(jr)
               t_h2j(jr)    = th2j(jr)*dr
               p_hij(jr)    = phij(jr)*dr
               w_lgth(jr)   = wlgth(jr)
               b_qjarr(jr)  = bqjarr(jr)
               d_rhoarr(jr) = drhoarr(jr)
            enddo
c
c                                scale the vector of parameters
            do i = 1, nxx
               li     = llxx(i)
               xa(li) = (xmax(li) + xmin(li))*.5d0
               xb(li) = 10.d0/(xmax(li) - xmin(li))
               xxf(i) = (xx(li) - xa(li))*xb(li)
            enddo
c                    ! copy variable model parameters into COMMON
            do jx = 1, nvar
               l_lxx(jx)  = llxx(jx)
               x_x(jx)    = xx(jx)
               x_a(jx)    = xa(jx)
               x_b(jx)    = xb(jx)
               x_delt(jx) = xdelt(jx)
            enddo
c
            open (iwr, file = oufile, status = 'old')
            do j = 1, 4000
               read (iwr, '(1x)', end = 63)
            enddo
63          continue
            backspace iwr
c
            fff  = func(nxx, xxf)
            write (iwr, '(a/a, g20.14)') trl, '# f(x0) = ', fff
            close (iwr)
            write (*, '(/a, e20.14, a, i4, a, i5, a, i3)')
     &       ' f = ', fff, '   iter = ', iter, 
     &       '   ncount = ', ncount, '  ier = ', ier
            ier = 0
c
c                                    ***     powell
            if (mv .eq. 'powell') then
               do i = 1, nxx
                  do j = 1, nxx
                     if (i .eq. j) then
                        parr(j, i) = dxopt
                     else
                        parr(j, i) = 0.d0
                     endif
                  enddo
               enddo
               call powell(xxf, parr, nxx, ftol, fff, iter)
            endif
c
            do i = 1, nxx
               li     = llxx(i)
               xx(li) = xxf(i)/xb(li) + xa(li)
            enddo
c
c
c                                               ! Output results, inversion
*     print *, char(7)
         open (iwr, file = oufile, status = 'old')
         do j = 1, 4000
            read (iwr, '(1x)', end = 65)
         enddo
65       continue
         backspace iwr
         write (iwr, '(a/a, e20.14, a, i4, a, i5, a, i3, a, f7.5)')
     &    trl, '# f = ', fff, '   iter = ', iter, 
     &    '   ncount = ', ncount, '  ier = ', ier 
c
         write (iwr, '(a/2a, i3, a/a)') trl, trl,
     &    dname, nref, ' reflectance values', trl
c
         write (iwr, '(a, 24x, 22a9)') trl, 'xmin', 'xmax', 
     &     'dx ', 'x0 ', 'x '
c
      write (iwr, '(a)') '#** Upper layer'
      jx = 28
      jt = 28
      j2 = 8
      do jr = 1, j2
         jx = jx + 1
         jt = jt + 1
         write (iwr, '(a, i3, 1x, a20, 23f9.3)') trl, 
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx), xe(jx), xx(jx)
      enddo
c
         write (iwr, '(a, 2x, a12, 8x, a12, i6, a)')
     & trl, 'Leaf model: ', lmod2, ncomp2, ' components'
      jt = 13
      do jc = 1, ncomp2
         jx = jx + 1
         jt = jt + 1
         write (iwr, '(a, 7x, 10a18)') trl, fcoef2(jc)
         write (iwr, '(a, i3, 1x, a20, 23f9.3)') trl, 
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx), xe(jx), xx(jx)
      enddo
c
c print LIBERTY parameters and PROSPECT parameter N
      if (lmod2 .eq. 'liberty') then
         jt = 22
         j1 = 46
         j2 = j1 + 2
      else
         jt = 25
         j1 = 49
         j2 = j1
      endif
      do jx = j1, j2
         jt = jt + 1
            write (iwr, '(a, i3, 1x, a20, 23f9.4)') trl, 
     &        jx, tex(jt), xmin(jx), xmax(jx), 
     &        xdelt(jx), xe(jx), xx(jx)
      enddo
c
      write (iwr, '(a)') '#** Lower layer'
      j1 = 50
      j2 = j1 + 6
      jt = 28
      do jx = j1, j2
         jt = jt + 1
         if (jt .eq. 32) jt = jt + 1
         write (iwr, '(a, i3, 1x, a20, 23f9.3)') trl, 
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx), xe(jx), xx(jx)
      enddo
c
         write (iwr, '(a, 2x, a12, 8x, a12, i6, a)')
     & trl, 'Leaf model: ', lmod1, ncomp1, ' components'
      jt = 13
      jx = j2
      do jc = 1, ncomp1
         jx = jx + 1
         jt = jt + 1
         write (iwr, '(a, 7x, 10a18)') trl, fcoef1(jc)
         write (iwr, '(a, i3, 1x, a20, 23f9.3)') trl,
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx), xe(jx), xx(jx)
      enddo
c
c print LIBERTY parameters and PROSPECT parameter N
      if (lmod1 .eq. 'liberty') then
         j1 = 67
         jt = 23
         j2 = j1 + 2
      else
         j1 = 69
         jt = 25
         j2 = j1
      endif
      do jx = j1, j2
         jt = jt + 1
            write (iwr, '(a, i3, 1x, a20, 23f9.4)') trl, 
     &        jx, tex(jt), xmin(jx), xmax(jx), 
     &        xdelt(jx), xe(jx), xx(jx)
      enddo
c
      jt = 36
      j1 = 70
      j2 = j1 + 3
      do jx = j1, j2
         jt = jt + 1
            write (iwr, '(a, i3, 1x, a20, 23f9.4)') trl,
     & jx, tex(jt), xmin(jx), xmax(jx), xdelt(jx), xe(jx), xx(jx)
      enddo
c
      close (iwr)
c
*     open (7, file = outx, status = 'unknown')
*        rewind 7
*        write (7, '(d27.17)') (xx(i), i = 1, nvar) 
*     close (7)
c
      goto 11
      endif
c
2     continue
      write (6, '(a, e20.14, a, i4, a, i5, a, i3)')
     &   ' f = ', fff, '   iter = ', iter, 
     &   '   ncount = ', ncount, '  ier = ', ier
c
c     Print input and final reflectance
      open (iwr, file = oufile, status = 'old')
      do j = 1, 4000
         read (iwr, '(1x)', end = 66)
      enddo
66    continue
      backspace iwr
      write (iwr, '(a/a)') trl, 
     &  '# Wl,  r_measured, STD,   r_simulated'
      do jr = 1, nref
         write (iwr, '(f6.1, 3f8.4)') 
     &      w_lgth(jr), bqjarr(jr), drhoarr(jr), b_simul(jr)
      enddo
      close (iwr)
      print *, char(7)
      stop 'ACRM'
c
      endif
c
      print *, char(7)
      stop 'ACRM'
c
88    continue
      jwl = nint((rlambda - (400.d0 - wlstep))/wlstep)
      print *, 'ACRM: Wavelength out of range'
      write (*, '(a, i6, g15.4)')
     &         'jw, lambda:', jwl, rlambda
      write (iwr, '(a, i6, g15.4 /)')
     &         'jw, lambda:', jwl, rlambda
      write (iwr, '(/)')
      close (iwr)
      stop 'ACRM'
97    write (*, '(3a)') 
     &      'Error in the file ', fname, 
     &      ' - reached the end of file'
      stop 'ACRM'
98    write (*, '(3a, i4)') 
     &      'Error in the file ', fname, 'line ', jl
      stop 'ACRM'
99    write (*, '(a)')  'File not found: ', fname
      stop 'ACRM'
      end
